<?php

namespace Egoi\Marketing\Block;

/**
 * Class Track
 *
 * @package Egoi\Marketing\Block
 */
class Track extends \Magento\Framework\View\Element\Template
{

    /**
     * @var \Magento\Sales\Model\CategoryFactory
     */
    protected $_categoryFactory;

    /**
     * @var \Magento\Sales\Model\ProductFactory
     */
    protected $_productFactory;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $_orderCollection;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $_helper;

    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $_registry;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkout;

    /**
     * @var \Egoi\Marketing\Model\AccountFactory
     */
    protected $_accountFactory;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;

    /**
     * Track constructor.
     *
     * @param \Magento\Framework\Registry                                $registry
     * @param \Magento\Checkout\Model\Session                            $checkout
     * @param \Magento\Customer\Model\Session                            $customerSession
     * @param \Egoi\Marketing\Helper\Data                                $helper
     * @param \Egoi\Marketing\Model\AccountFactory                       $accountFactory
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollection
     * @param \Magento\Catalog\Model\ProductFactory                      $productFactory
     * @param \Magento\Catalog\Model\CategoryFactory                     $categoryFactory
     * @param \Magento\Framework\View\Element\Template\Context           $context
     */
    public function __construct(
        \Magento\Framework\Registry $registry,
        \Magento\Checkout\Model\Session $checkout,
        \Magento\Customer\Model\Session $customerSession,
        \Egoi\Marketing\Helper\Data $helper,
        \Egoi\Marketing\Model\AccountFactory $accountFactory,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollection,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Catalog\Model\CategoryFactory $categoryFactory,
        \Magento\Framework\View\Element\Template\Context $context
    )
    {

        $this->_registry = $registry;
        $this->_customerSession = $customerSession;
        $this->_accountFactory = $accountFactory;
        $this->_checkout = $checkout;
        $this->_helper = $helper;
        $this->_orderCollection = $orderCollection;
        $this->_productFactory = $productFactory;
        $this->_categoryFactory = $categoryFactory;

        parent::__construct($context);
    }

    /**
     * @return int|null
     */
    public function getCustomerId()
    {

        return $this->_customerSession->getCustomerId();
    }

    /**
     * @return mixed
     */
    public function getAccount()
    {

        return $this->_accountFactory->create()->getAccount();
    }

    /**
     * @return string|void
     */
    public function _getOrdersTrackingCode()
    {

        $orderIds = $this->getOrderIds();
        if (empty($orderIds) || !is_array($orderIds)) {
            return false;
        }

        $collection = $this->_orderCollection->create()
                                             ->addFieldToFilter('entity_id', ['in' => $orderIds]);
        $result = [];

        /** @var \Magento\Sales\Model\Order $order */
        foreach ($collection as $order) {
            foreach ($order->getAllVisibleItems() as $item) {

                if ($item->getQtyOrdered()) {
                    $qty = number_format($item->getQtyOrdered(), 0, '.', '');
                } else {
                    $qty = '0';
                }

                $result[] = sprintf(
                    "_egoiaq.push(['addEcommerceItem',  '%s', '%s', '%s', %s, %s]);",
                    $this->escapeJsQuote($item->getSku()),
                    $this->escapeJsQuote($item->getName()),
                    '',
                    number_format($item->getBasePrice(), 2),
                    $qty
                );

            }

            $subtotal = number_format(
                $order->getGrandTotal() - $order->getShippingAmount() - $order->getShippingTaxAmount(),
                2
            );

            $result[] = sprintf(
                "_egoiaq.push(['trackEcommerceOrder', '%s', '%s','%s', '%s', '%s']);",
                $order->getIncrementId(),
                number_format($order->getBaseGrandTotal(), 2),
                $subtotal,
                number_format($order->getBaseTaxAmount(), 2),
                number_format($order->getBaseShippingAmount(), 2)
            );

        }

        return implode("\n", $result);
    }

    public function _getEcommerceCartUpdate()
    {

        $cart = $this->_checkout->getQuote()->getAllVisibleItems();

        foreach ($cart as $cartitem) {

            //get category name
            $_product = $cartitem->getProduct();
            $cats = $_product->getCategoryIds();

            $categoryName = '';

            if (isset($cats)) {
                $category_id = end($cats);
                $category = $this->_categoryFactory->create()->load($category_id);
                $categoryName = $this->escapeJsQuote($category->getName());
            }

            $productName = $this->escapeJsQuote($cartitem->getName());

            echo sprintf(
                "_egoiaq.push(['addEcommerceItem', '%s', '%s', '%s', %s, %s]);",
                $cartitem->getSku(),
                $productName,
                $categoryName,
                number_format($cartitem->getPrice(), 2),
                (int) $cartitem->getQty()
            );

            echo "\n";
        }

        //total in cart
        $grandTotal = number_format($this->_checkout->getQuote()->getGrandTotal(), 2);
        if ($grandTotal == 0) {
            echo '';
        } else {
            echo '_egoiaq.push(["trackEcommerceCartUpdate", "' . $grandTotal . '"]);';
        }
        echo "\n";
    }

    public function _getProductPageview()
    {

        $currentProduct = $this->_registry->registry('current_product');
        $currentCategory = $this->_registry->registry('current_category');

        if (!($currentProduct instanceof \Magento\Catalog\Model\Product)) {
            return;
        }

        $categoryName = '';
        if ($currentCategory && $currentCategory->getId()) {
            $categoryName = $this->escapeJsQuote($currentCategory->getName());
        }

        $productName = $this->escapeJsQuote($currentProduct->getName());

        echo sprintf(
            "_egoiaq.push(['setEcommerceView', '%s', '%s', '%s', '%s']);",
            $currentProduct->getSku(),
            $productName,
            $categoryName,
            number_format($currentProduct->getPrice(), 2)
        );

        $this->_registry->unregister('current_category');
    }

    public function _getCategoryPageview()
    {

        $currentCategory = $this->_registry->registry('current_category');

        if (!($currentCategory instanceof \Magento\Catalog\Model\Category)) {
            return;
        }

        echo sprintf(
            "_egoiaq.push(['setEcommerceView', %s, %s, '%s']);",
            false,
            false,
            $this->escapeJsQuote($currentCategory->getName())
        );

        $this->_registry->unregister('current_product');
    }

    /**
     * @return mixed|string
     */
    protected function _toHtml()
    {

        if (!$this->_helper->isTrackAvailable($this->_storeManager->getStore()->getId())) {
            return '';
        }

        return str_replace("\n\n", "\n", parent::_toHtml());
    }

}