<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2018 E-Goi - http://e-goi.com
 */

namespace Egoi\Marketing\Helper;

use Magento\Framework\App\Helper\Context;

/**
 * Class Data
 *
 * @package Egoi\Marketing\Helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{

    const XML_PATH_ACTIVE = 'egoi/info/analytics';

    const TRANSACTIONAL_SERVER = 'bo51.e-goi.com';

    const TRANSACTIONAL_PORT = 587;

    const TRANSACTIONAL_AUTH = 'login';

    const TRANSACTIONAL_SSL = 'TLS';

    /**
     * @var \Egoi\Marketing\Model\ResourceModel\Subscriber\CollectionFactory
     */
    protected $_subscriberCollection;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * Data constructor.
     *
     * @param \Magento\Checkout\Model\Session                                  $checkoutSession
     * @param \Magento\Customer\Model\Session                                  $customerSession
     * @param Context                                                          $context
     * @param \Egoi\Marketing\Model\ResourceModel\Subscriber\CollectionFactory $subscriberCollection
     */
    public function __construct(
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Customer\Model\Session $customerSession,
        Context $context,
        \Egoi\Marketing\Model\ResourceModel\Subscriber\CollectionFactory $subscriberCollection
    )
    {

        parent::__construct($context);

        $this->_customerSession = $customerSession;
        $this->_checkoutSession = $checkoutSession;
        $this->_subscriberCollection = $subscriberCollection;
    }

    /**
     *
     * @param mixed $store
     *
     * @return bool
     */
    public function isTrackAvailable($store)
    {

        return $this->scopeConfig->isSetFlag(
            self::XML_PATH_ACTIVE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @return bool|\Magento\Framework\DataObject
     */
    public function loadSubscriber()
    {

        $fieldValue = $this->getCustomerEmail();
        $fieldName = 'email';

        if (!$fieldValue) {
            $fieldValue = $this->getCustomerId();
            $fieldName = 'customer_id';
        }
        if (!$fieldValue) {
            return false;
        }

        $col = $this->_subscriberCollection->create()
                                           ->addFieldToFilter($fieldName, $fieldValue);

        return $col->getSize() > 0 ? $col->getFirstItem() : false;
    }

    /**
     * @return bool|string
     */
    public function getCustomerEmail()
    {

        $quote = $this->_checkoutSession->getQuote();
        if ($this->_customerSession->getCustomer() && $this->_customerSession->getCustomer()->getEmail()) {
            return $this->_customerSession->getCustomer()->getEmail();
        } elseif ($quote->getCustomerEmail()) {
            return $quote->getCustomerEmail();
        } elseif ($quote->getBillingAddress() && $quote->getBillingAddress()->getEmail()) {
            return $quote->getBillingAddress()->getEmail();
        }

        return false;
    }

    /**
     * @return bool|int|mixed|null
     */
    public function getCustomerId()
    {

        $quote = $this->_checkoutSession->getQuote();
        if ($quote->getCustomerId()) {
            return $quote->getCustomerId();
        } elseif ($this->_customerSession->getCustomerId()) {
            return $this->_customerSession->getCustomerId();
        }

        return false;
    }

    /**
     * @return string
     */
    public function getSmtpServer()
    {

        return self::TRANSACTIONAL_SERVER;
    }

    /**
     * @return array
     */
    public function getSmtpDetails()
    {

        return [
            'auth'     => self::TRANSACTIONAL_AUTH,
            'ssl'      => self::TRANSACTIONAL_SSL,
            'port'     => self::TRANSACTIONAL_PORT,
            'username' => $this->scopeConfig->getValue('egoi/transactional/username'),
            'password' => $this->scopeConfig->getValue('egoi/transactional/password'),
        ];
    }

    /**
     *
     * @return \Zend_Mail_Transport_Smtp
     */
    public function getSmtpTransport()
    {

        $config = ['auth' => self::TRANSACTIONAL_AUTH, 'port' => self::TRANSACTIONAL_PORT];

        $config['ssl'] = self::TRANSACTIONAL_SSL;

        $config['username'] = $this->scopeConfig->getValue('egoi/transactional/username');
        $config['password'] = $this->scopeConfig->getValue('egoi/transactional/password');

        return new \Zend_Mail_Transport_Smtp(self::TRANSACTIONAL_SERVER, $config);
    }

    /**
     * @param string $file
     *
     * @return \Zend\Log\Logger
     */
    public function getLogger($file = '')
    {

        $file = trim('egoi-' . $file, '-');

        $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/' . $file . '.log');
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);

        return $logger;
    }
}
