<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2018 E-Goi - http://e-goi.com
 */

namespace Egoi\Marketing\Plugin;

use Magento\Framework\Mail\TransportInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * Class TransportPlugin
 *
 * @package Egoi\Marketing\PLugin
 */
class TransportPlugin
{

    /**
     * @var ScopeConfigInterface
     */
    protected $_scope;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $_helper;

    /**
     * TransportPlugin constructor.
     *
     * @param \Egoi\Marketing\Helper\Data $helper
     * @param ScopeConfigInterface        $scopeConfigInterface
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data $helper,
        ScopeConfigInterface $scopeConfigInterface
    )
    {

        $this->_scope = $scopeConfigInterface;
        $this->_helper = $helper;

    }

    /**
     * @param TransportInterface $subject
     * @param \Closure           $proceed
     *
     * @return mixed
     */
    public function aroundSendMessage(
        TransportInterface $subject,
        \Closure $proceed
    )
    {

        if (!$this->_scope->isSetFlag('egoi/transactional/enable')) {

            return $proceed();

        } else {
            try {
                $transport = new \Zend_Mail_Transport_Smtp(
                    $this->_helper->getSmtpServer(), $this->_helper->getSmtpDetails()
                );

                if (method_exists($subject, 'getMessage')) {

                    if ($domain = $this->_scope->getValue('egoi/transactional/domain')) {
                        $subject->addHeader('X-Domain', $domain);
                    }

                    $transport->send($subject->getMessage());

                } else {
                    $reflection = new \ReflectionClass($subject);
                    $property = $reflection->getProperty('_message');
                    $property->setAccessible(true);
                    $transport->send($property->getValue($subject));

                }
            } catch (\Exception $e) {
            }

        }

    }
}
